#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <string.h>
#include <ctype.h>

#include "wordblox.h"
#include "constantstrings.h"

/* export the puzzle to a png file */
void export_puzzle (Puzzle *p)
{
	if (p->grid_frozen == false)
	{
		printf (UNFROZEN_GRID);
		char ch = getchar ();
		return;
	}
	char fname[256];
	printf (INPUT_FILE);
	fgets (fname, 256, stdin);
	char* filename = strtok (fname, "\n");

	printf (INPUT_EXPORT_ANSWERS);
	char ans[3];
	fgets (ans, 3, stdin);
	bool solution;
	solution = (toupper (ans[0]) == 'Y') ? true : false;
		
	export_grid_image (p, filename, solution);
	printf (FILE_SAVED);
	char ch = getchar ();
}

/* set clue for a word - only for frozen grid */
void set_clue_word (Puzzle *p, enum ORIENTATION orient)
{
	print_puzzle (p);
	if (p->grid_frozen == false)
	{
		printf (UNFROZEN_GRID);
		char ch = getchar ();
		return;
	}
	int index;
	String clue; 
	printf (INPUT_INDEX);
	index = get_num ();
	printf (INPUT_CLUE);
	fgets (clue, MAX_CLUE_LENGTH, stdin);
	char* cl = strtok (clue, "\n");
	
	bool res;
	res = set_clue (p, cl, index, orient);

	if (res == false)
	{
		printf (NO_WORD_INDEX);
		char ch = getchar ();
	}
}

/* clear a cell in the grid */
void clear_cell (Puzzle *p)
{
	print_puzzle (p);
	if (p->grid_frozen == true) 
	{
		printf (FROZEN_GRID);
		char ch = getchar ();
		return;
	}
	int row, col;
	printf (INPUT_ROW);
	row = get_num (); 
	printf (INPUT_COL);
	col = get_num ();
	if (row >= p->grid_size || col >= p->grid_size)
	{
		printf (EXCEED_GRID_SIZE);
		char ch = getchar ();
		return;
	}
	p->chars[row][col] = ' ';
	print_puzzle (p);
	
	char ch = getchar ();
}

/* add a down word to the grid */
void add_down_word (Puzzle *p)
{
	print_puzzle (p);
	if (p->grid_frozen == true) 
	{
		printf (FROZEN_GRID);
		char ch = getchar ();
		return;
	}
	char wd[MAX_PUZZLE_SIZE];
	int row, col;
	printf (INPUT_WORD);
	fgets (wd, MAX_PUZZLE_SIZE, stdin);
	char *word = strtok (wd, "\n");
	if (! is_valid_word (word))
	{
		printf (INVALID_WORD);
		char ch = getchar ();
		return;
	}
	printf (INPUT_ROW);
	row = get_num ();
	printf (INPUT_COL);
	col = get_num ();
	if (row >= p->grid_size || col >= p->grid_size) 
	{
		printf (EXCEED_GRID_SIZE);
		char ch = getchar ();
		return;
	}
	if (strlen (word) > (p->grid_size - row))
	{
		printf (WORD_TOO_LONG);
		char ch = getchar ();
		return;
	}
	
	for (int i = row; i < row + strlen(word); i ++)
		p->chars[i][col] = toupper(word[i - row]);
	
	print_puzzle (p);
	char ch = getchar ();
}

/* add an across word to the grid */
void add_across_word (Puzzle *p)
{
	print_puzzle (p);
	if (p->grid_frozen == true) 
	{
		printf (FROZEN_GRID);
		char ch = getchar ();
		return;
	}
	char wd[MAX_PUZZLE_SIZE];
	int row, col;	
	printf (INPUT_WORD);
	fgets (wd, MAX_PUZZLE_SIZE, stdin);
	char *word = strtok (wd, "\n");
	if (! is_valid_word (word))
	{
		printf (INVALID_WORD);
		char ch = getchar ();
		return;
	}	
	printf (INPUT_ROW);
	row = get_num ();
	printf (INPUT_COL);
	col = get_num ();
	if (row >= p->grid_size || col >= p->grid_size)
	{
		printf (EXCEED_GRID_SIZE);
		char ch = getchar ();
		return;
	}
	
	if (strlen (word) > (p->grid_size - col))
	{
		printf (WORD_TOO_LONG);
		char ch = getchar ();
		return;
	}
	
	for (int i = col; i < col+strlen (word); i ++)
		p->chars[row][i] = toupper(word[i - col]);
		
	print_puzzle (p);
	char ch = getchar ();
}
/* confirm exit */
bool confirm_exit ()
{
	printf (INPUT_CONFIRM_EXIT);
	char res[3];
	fgets (res, 3, stdin);
	if (toupper(res[0]) == 'Y')
		return true;
	else
		return false;
} 

/* main loop for the puzzle editor */
void puzzle_editor_loop (Puzzle *p, const char *filename) 
{
	bool loop = true;
	while (loop) 
	{
		print_menu (WHITE, RED, PUZZLE_MENU_TITLE, PUZZLE_EDIT_MENU, 11, 50);
		printf (INPUT_CHOICE);
		int ch = get_num ();
		switch (ch)
		{
			case 1: print_puzzle (p);
					char ch = getchar ();
					break;
			case 2: add_across_word (p);
					break;
			case 3: add_down_word (p);
					break;
			case 4: clear_cell (p);
					break;
			case 5: freeze_puzzle (p);
					print_puzzle (p);
					ch = getchar ();
					break;
			case 6: unfreeze_puzzle (p);
					print_puzzle (p);
					ch = getchar ();
					break;
			case 7: set_clue_word (p, ACROSS);
					break;
			case 8: set_clue_word (p, DOWN);
					break;
			case 9: save_puzzle (p, filename);
					printf ("%s\n",FILE_SAVED);
					ch = getchar ();
					break;
			case 10: export_puzzle (p);
					 break;
			case 11: loop = !confirm_exit ();
					 break;
		}
	}
}

/* open an existing puzzle */
void open_puzzle () 
{
	Puzzle p;
	printf (INPUT_FILE);
	char fname[256];
	fgets(fname, 256, stdin);
	char* filename = strtok (fname, "\n");
	p = load_puzzle (filename);
	puzzle_editor_loop (&p, filename);
}

/* create a new blank puzzle */
void new_puzzle () 
{
	Puzzle p; 
	printf (INPUT_FILE);
	char fname[256];
	fgets (fname, 256, stdin);
	char* filename = strtok (fname, "\n"); 
	printf (INPUT_GRID_SIZE);
	int size;
	size = get_num ();
	if (size > MAX_PUZZLE_SIZE) 
	{ 
		printf (EXCEED_MAX_GRID_SIZE);
		char c = getchar (); 
	}
	else
	{
		init_puzzle (&p, size);
		puzzle_editor_loop (&p, filename);
	}
}

/* The main loop of the program */
int main_loop () 
{
	/* Print the main menu */
	while (1)
	{
		print_menu (WHITE, BLUE, MAIN_MENU_TITLE, MAIN_MENU, 3, 50);
		printf (INPUT_CHOICE);
		int ch = get_num ();
		switch (ch)
		{
			case 1: new_puzzle ();
					break;
			case 2: open_puzzle ();
					break;
			case 3: exit (0);
		}
	}
}

int main (int argc, char* argv[]) 
{
	if (argc >= 2)
	{
		Puzzle p;
		switch (argc)
		{
			case 2 : p = load_puzzle (argv[1]);
					 puzzle_editor_loop (&p, argv[1]);
					 break;
			case 4 : if (strcmp (argv[2], "new") == 0)
					 {
					 	int grid_size = atoi (argv[3]);
					 	init_puzzle (&p, grid_size);
					 	puzzle_editor_loop (&p, argv[1]);
					 	break;
					 }
			default: fprintf (stderr, USAGE_LINE_1, argv[0]);
					 fprintf (stderr, USAGE_LINE_2);
					 fprintf (stderr, USAGE_LINE_3);
					 exit (3);
		}
	}
	return (main_loop ());
}
